﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ObserverPatternDemo
{
    public class WareHouse
    {
        public WareHouse()
        {
            ObservingStores = new List<Store>();
        }

        public void AttachObserver( Store store )
        {
            if( !ObservingStores.Contains(store))
            {
                ObservingStores.Add(store);
            }
        }

        public void DetachObserver(Store store)
        {
            if (ObservingStores.Contains(store))
            {
                ObservingStores.Remove(store);
            }
        }

        public void RemoveProductFromInventory( string sku, Int32 amountToRemove )
        {
            NotifyObservingStoresOfInvetoryRemoval(sku, amountToRemove);
        }

        private void NotifyObservingStoresOfInvetoryRemoval(string sku, int amountToRemove)
        {
            foreach (var store in ObservingStores)
            {
                store.RemoveInvetoryForSku(sku, amountToRemove);
            }
        }

        public IList<Store> ObservingStores { get; private set; }
    }

    public abstract class Store
    {
        public Store( string storeName)
        {
            StoreName = storeName;
            Invenotry = new Dictionary<string, int>();
            PopulateInventory();
        }

        protected abstract void PopulateInventory();

        public void RemoveInvetoryForSku(string sku, int amountToRemove)
        {
            if( Invenotry.ContainsKey(sku))
            {
                var currentInventory = Invenotry[sku];
                Invenotry[sku] = currentInventory - amountToRemove;
            }
        }

        public string StoreName { get; private set; }
        public Dictionary<string, Int32> Invenotry { get; set; }

        public string InventoryForSku(string sku)
        {
            if ( Invenotry.ContainsKey(sku))
            {
                return Invenotry[sku].ToString();
            }

            return 0.ToString();
        }
    }

    public class WestHollywoodStore : Store
    {
        public WestHollywoodStore(string storeName)
            : base(storeName)
        {
        }

        protected override void PopulateInventory()
        {
            Invenotry.Add("abc", 10);
            Invenotry.Add("xyz", 5);
            Invenotry.Add("333", 7);
            Invenotry.Add("123", 7);
        }
    }

    public class ChicagoStore : Store
    {
        public ChicagoStore(string storeName)
            : base(storeName)
        {
        }

        protected override void PopulateInventory()
        {
            Invenotry.Add("abc", 12);
            Invenotry.Add("xyz", 4);
            Invenotry.Add("333", 5);
            Invenotry.Add("123", 6);
        }
    }

}
